@@ -79,7 +79,7 @@ class Timeline(urwid.Columns):
         return urwid.ListBox(walker)
 
     def build_list_item(self, status):
-        item = StatusListItem(status, self.tui.args.relative_datetimes)
+        item = StatusListItem(status, self.tui.options.relative_datetimes)
         urwid.connect_signal(item, "click", lambda *args:
             self.tui.show_context_menu(status))
         return urwid.AttrMap(item, None, focus_map={
@@ -95,19 +95,19 @@ class Timeline(urwid.Columns):
             return None
 
         poll = status.original.data.get("poll")
-        show_media = status.original.data["media_attachments"] and self.tui.media_viewer
+        show_media = status.original.data["media_attachments"] and self.tui.options.media_viewer
 
         options = [
             "[A]ccount" if not status.is_mine else "",
             "[B]oost",
             "[D]elete" if status.is_mine else "",
+            "[E]dit" if status.is_mine else "",
             "B[o]okmark",
             "[F]avourite",
             "[V]iew",
             "[T]hread" if not self.is_thread else "",
             "L[i]nks",
             "[M]edia" if show_media else "",
-            self.tui.media_viewer,
             "[R]eply",
             "[P]oll" if poll and not poll["expired"] else "",
             "So[u]rce",
@@ -190,6 +190,11 @@ class Timeline(urwid.Columns):
                 self.tui.show_delete_confirmation(status)
             return
 
+        if key in ("e", "E"):
+            if status.is_mine:
+                self.tui.async_edit(status)
+            return
+
         if key in ("f", "F"):
             self.tui.async_toggle_favourite(self, status)
             return
@@ -389,6 +394,8 @@ class StatusDetails(urwid.Pile):
 
         yield ("pack", urwid.Text([
             ("status_detail_timestamp", f"{status.created_at.strftime('%Y-%m-%d %H:%M')} "),
+            ("status_detail_timestamp",
+             f"(edited {status.edited_at.strftime('%Y-%m-%d %H:%M')}) " if status.edited_at else ""),
             ("status_detail_bookmarked" if status.bookmarked else "dim", "b "),
             ("dim", f"⤶ {status.data['replies_count']} "),
             ("highlight" if status.reblogged else "dim", f"♺ {status.data['reblogs_count']} "),
@@ -443,7 +450,7 @@ class StatusDetails(urwid.Pile):
 
 class StatusListItem(SelectableColumns):
     def __init__(self, status, relative_datetimes):
-        edited_at = status.data.get("edited_at")
+        edited_at = status.original.edited_at
 
         # TODO: hacky implementation to avoid creating conflicts for existing
         # pull reuqests, refactor when merged.
@@ -457,7 +464,7 @@ class StatusListItem(SelectableColumns):
         favourited = ("highlight", "★") if status.original.favourited else " "
         reblogged = ("highlight", "♺") if status.original.reblogged else " "
         is_reblog = ("dim", "♺") if status.reblog else " "
-        is_reply = ("dim", "⤶") if status.original.in_reply_to else " "
+        is_reply = ("dim", "⤶ ") if status.original.in_reply_to else "  "
 
         return super().__init__([
             ("pack", SelectableText(("status_list_timestamp", created_at), wrap="clip")),
